
BEGIN {
  unless ($ENV{RELEASE_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for release candidate testing');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::NoTabsTests 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/Config/Onion.pm',
    'lib/Config/Onion/Simple.pm',
    't/00-load.t',
    't/01-defaults.t',
    't/02-load.t',
    't/03-mixed.t',
    't/04-simple.t',
    't/conf/basic.yml',
    't/conf/basic2.zzz',
    't/conf/prefix.yml',
    't/conf/withlocal.local.yml',
    't/conf/withlocal.yml',
    't/release-eol.t',
    't/release-no-tabs.t'
);

notabs_ok($_) foreach @files;
done_testing;
